#!/usr/bin/env python3
# Authors: Mabel Villalba Jimenez <mabelvj@gmail.com>,
#          Emilio Molina Martinez <emilio.mol.mar@gmail.com>
# License: GPLv3

# Multiple formats for each line.
from bokeh.models.widgets import Slider
from bokeh.models.widgets import PreText

import re
import copy
import pandas as pd
import numpy as np

try:
    from ..stocksdashboard.stocksdashboard import StocksDashboard
except Exception as excinfo:
    print(str(excinfo))
    from stocksdashboard.stocksdashboard import StocksDashboard

try:
    from .formatter import Formatter
except Exception as excinfo:
    print(str(excinfo))
    from formatter import Formatter

class DashboardWithWidgets:
    def __init__(self, sdb, sliders_params, signals_expressions):
        global _widget_type
        _widget_type = (Slider, PreText)
        self.sliders = {}
        self.pretext = {}
        assert(isinstance(sdb, StocksDashboard))
        self.sdb = sdb
        self.sliders_params = sliders_params
        self.__check_sliders()
        self.signals_expressions = signals_expressions

    def __check_sliders(self):
        assert(self.sliders_params is not None)
        assert(isinstance(self.sliders_params, dict)), (
            "'sliders_params' should be" +
            " a dictionary with format, i.e.: " +
            "\{name_of_var: \{'title' : 'EMA'," +
            "'params': \{value':20, " +
            "'start':2, 'end':252, 'step':5\}\}\}")
        for k, v in list(self.sliders_params.items()):
            assert isinstance(v, dict), (
                "%s should contain a dict" % k +
                " as value. Found type %s: %s" % (type(v), v))
            assert(all([field in list(v.keys())
                        for field in ('title', 'params')])), (
                "'sliders_params should contain a dict containing:" +
                " ('title', 'expression', 'params'). " +
                "Found: %s" % v.keys())
            # "'expression': {'EMA': 'STOCK.ewm(span=w, min_periods=1,'" +
            # "'adjust=True,ignore_na=False)' " +
            # "'.mean()'\}"

    def create_sliders(self):
        # Sliders
        sliders = {}
        for name, args in list(self.sliders_params.items()):
            sliders[name] = Slider(title=args['title'], **args['params'])
        self.sliders = sliders
        return sliders

    @staticmethod
    def replace_var(expr, varname, dict_name):
        return expr.replace(varname, "%s[%s]" % (dict_name, varname))

    def update_expression(self, var_dict, expression, dict_name,
                          expr=["(?<=\()(\w+)(?=\))",
                                "(?<==)(\w+)+(?=,)"]):
        global _widget_type
        expression_temp = copy.deepcopy(expression)
        replaced = set()
        for e in expr:
            for word in re.findall(e, expression_temp):
                if word in list(var_dict.keys()) and word not in replaced:
                    word_pattern = e.replace('\\w+', '\\b' + word)
                    word_replacement = "%s['%s']" % (dict_name, word)
                    if isinstance(var_dict[word], _widget_type):
                        word_replacement = word_replacement + '.value'
                    expression_temp = re.sub(
                        word_pattern, word_replacement, expression_temp)
                    replaced.add(word)
        return expression_temp

    def _format_signal_expressions(self, data_temp):
        signals_expressions_formatted = {}
        for signal_name, expr in list(self.signals_expressions.items()):
            expression_temp = copy.deepcopy(expr)
            expression_temp = self.update_expression(
                data_temp,
                expression_temp, 'data_temp',
                ["(\w+)(?=\W+)", "(?<=\W)(\w+)(?=\W+)", "(?<=\()(\w+)(?=\))",
                 "(?<==)(\w+)+(?=,)"])
            expression_temp = self.update_expression(
                self.sliders,
                expression_temp, 'self.sliders',
                ["(\w+)(?=\W+)", "(?<=\()(\w+)(?=\))", "(?<==)(\w+)+(?=,)"])
            signals_expressions_formatted[signal_name] = copy.deepcopy(
                expression_temp)
        self.signals_expressions_formatted = copy.deepcopy(
            signals_expressions_formatted)

        return signals_expressions_formatted

    def update_data(self, attrname, old, new):
        sliders_values = {}
        data_temp = {}
        result = {}
        for k, v in list(self.sliders.items()):
            sliders_values[k] = v.value
        for i, __data_source in enumerate(self.sdb.datasources):
            for name in list(__data_source.data.keys()):
                if re.findall("\(\w+\)", name):
                    raise(ValueError("Variable should not contain " +
                                     "plain parentheses. "
                                     "If included use '\(' and '\)'." +
                                     "Found: %s" % name))
                if len(__data_source.data[name]) > 1:
                    data_temp[name] = pd.Series(__data_source.data[name],
                                                index=__data_source.data['x'])
                else:
                    data_temp[name] = __data_source.data[name]
        if not hasattr(self, 'signal_expressions_formatted'):
            self._format_signal_expressions(data_temp)
        for signal_name, expr in list(self.signals_expressions.items()):
            result[signal_name] = eval(
                self.signals_expressions_formatted[signal_name])

        for i, __data_source in enumerate(self.sdb.datasources):
            for name in result:
                if name in __data_source.data:
                    (__data_source.data['x'],
                     __data_source.data[name]
                     ) = Formatter._get_x_y(result[name])

    def widget_on_change(self):
        list_of_widgets = list(self.sliders.values())
        for _widget in list_of_widgets:
            # print(w)
            if isinstance(_widget, PreText):
                attribute_name = 'text'
            else:
                attribute_name = 'value'
            _widget.on_change(attribute_name, self.update_data)



import os


def configuration(parent_package='', top_path=None):
    from numpy.distutils.misc_util import Configuration

    libraries = []
    if os.name == 'posix':
        libraries.append('m')

    config = Configuration('stocksdashboard', parent_package, top_path)

    # the following packages depend on cblas, so they have to be build
    # after the above.
    # config.add_subpackage('utils')

    # add the test directory
    config.add_subpackage('tests')

    return config


if __name__ == '__main__':
    from numpy.distutils.core import setup
    setup(**configuration(top_path='').todict())



#!/usr/bin/env python3
# Authors: Mabel Villalba Jimenez <mabelvj@gmail.com>,
#          Emilio Molina Martinez <emilio.mol.mar@gmail.com>
# License: GPLv3

import numpy as np
import pandas as pd
import copy
# try:
#     from .stocksdashboard import convert_to_datetime
# except Exception as excinfo:
#     print(str(excinfo))
#     from stocksdashboard import convert_to_datetime


def convert_to_datetime(x):
    return np.array(x, dtype=np.datetime64)


class Formatter():

    """
        Formats data to valid data accepted by StocksDashboard.

        Data should be contained in dicts, being the key of the dict the title
        of each plot element:
            - {'stocks': {'AAPL': AAPL, 'GOOG': GOOG,
                     'IBM': IBM, 'MSFT': MSFT, ...},
               'avg': {'AAPL_avg': aapl_avg, ...}}

        Each of the values of the input data dictionary
        should have either of the following formats:
            - dict of dicts:
                - must contains at least one ts column to be plotted
                  (set int :meth:StocksDashboard.build_dashboard()`:
                  {'GOOGL' : {'dates': ..., 'adj_close': ...}}
            - dict of pd.Series
                - index will be used as dates
            - dict of pandas.DataFrame:
                - index will be used as dates
                - data from `column` in StocksDashboard.plot_stock()
                  will be used.
            - dict of np.array
                - index will be a list of numbers
                - no column will be selected.
            - list of dicts with keys ('dates' and column):
                [{'dates': ..., 'adj_close': ...},
                 {'dates': ..., 'adj_close': ...}]
            - list of pd.Series / pd.DataFrame.
    """

    def __init__(self):
        self.name = None

    def _format(self, data):
        """
            Format data for plotting.
        """

        if isinstance(data, list):
            return self.__process_list(data)

        elif isinstance(data, dict):
            return self.__process_dict(data)

        elif isinstance(data, (pd.Series, pd.DataFrame, np.ndarray)):
            # Convert to a list of, at least,
            # one element, to be able to iterate.
            if not isinstance(data, pd.DataFrame):
                data = [pd.DataFrame(data)]
            else:
                data = [data]
        else:
            raise(TypeError("Data type is not valid."))

    @staticmethod
    def __is_valid_type(data):
        _valid_types = (pd.DataFrame, pd.Series, list, dict, np.ndarray)

        if not (data is None or isinstance(data, _valid_types)):

            raise(ValueError("Inappropiate value " +
                             "of 'data' : %s. " % data +
                             "Expected pandas.DataFrame, pandas.Series, " +
                             "or list of pandas objects."))
        return True

    def reformat_x_list(self, data):
        # if all are pd.DataFrame or pd.Series -> merge indices!!
        if any([isinstance(d, (pd.Series, pd.DataFrame)) for d in data]):
            # return each of the dataframes with the merged indices
            return [s.to_frame()
                    for c, s in pd.concat(copy.deepcopy(data),
                                          axis=1).iteritems()]

    def reformat_x_dict(self, data):
        # if all are pd.DataFrame or pd.Series -> merge indices!!
        if any([isinstance(d, (pd.Series, pd.DataFrame))
                for d in list(data.values())]):
            # return each of the dataframes with the merged indices
            df_total = pd.concat(copy.deepcopy(data), axis=1)
            columns = list(data.keys())
            return {c: df_total.loc[:, c] for c in columns}

    def __process_list(self, data):
        """
            Format list to valid type.
        """
        # list of dicts
        if all([isinstance(d, dict) for d in data]):
            result = [pd.DataFrame.from_dict(d) for d in data]
            # Has 'date' as a column -> format to common date
            if all(['date' in df for df in result]):
                result = [df.set_index('date') for df in result]
            n = len(result[0])
            # if not all([len(df) == n for df in result]):
            result = self.reformat_x_list(result)
            return result
        # data is list of np.ndarray
        elif all([isinstance(d, np.ndarray) for d in data]):
            result = data
            n = len(result[0])
            assert(all([len(arr) == n for arr in result])), (
                "All elements in a list of np.ndarray should have " +
                "the same length")
            return result
        # data is list of pd.Series, pd.DataFrame[
        elif any([isinstance(d, (pd.Series, pd.DataFrame)) for d in data]):
            # Check all index are same type
            assert all([isinstance(d.index, type(data[0].index))
                        for d in data]), (
                "All indices in a list of pd.Series or pd.DataFrames " +
                "should have the same type.")
            result = self.reformat_x_list(data)
            return result
        else:
            raise(TypeError("Data is not valid. " +
                            "If 'list' elements should be:" +
                            " dicts, pd.Series or pd.DataFrame. " +
                            "Found : %s" % [type(d) for d in data]))

    def __process_dict(self, _data):
        """
            Format dictionary to valid type.
        """
        data = copy.deepcopy(_data)
        self.names = list(data.keys())
        # dict of dicts
        if all([isinstance(d, dict) for d in list(data.values())]):
            result = {k: pd.DataFrame.from_dict(
                d) for k, d in list(data.items())}
            # Has 'date' as a column -> format to common date
            if all(['date' in v for k, v in result.items()]):
                result = {k: df.set_index('date')
                          for k, df in list(result.items())}
            result = self.reformat_x_dict(result)
            return list(result.values())
        # data is list of np.ndarray
        elif all([isinstance(d, np.ndarray) for d in list(data.values())]):
            result = data
            n = len(list(result.values())[0])
            assert(all([len(arr) == n for arr in list(result.values())])), (
                "All elements in a list of np.ndarray should have " +
                "the same length")
            return list(result.values())
        # dict of dataframes, pd.Series
        elif any([isinstance(d, (pd.Series, pd.DataFrame))
                  for d in list(data.values())]):
            assert all([isinstance(d.index, type(list(data.values())[0].index))
                        for d in list(data.values())]), (
                "All indices in a dict of pd.Series or pd.DataFrames " +
                "should have the same type.")

            result = self.reformat_x_dict(data)
            return list(result.values())
        else:
            raise(TypeError("Data is not valid. " +
                            "Found dict containing objects" +
                            " of tpye %s." % [type(d) for d in list(data)] +
                            "Please convert to format" +
                            " \{'name': \{'date': ..., 'values': ...\}\}"
                            "or {'name': {'date': pd.DataFrame}."))

    @staticmethod
    def _get_x_y(data, column=None):
        """
            Get the x and y coordinates to be plotted in the line graph.

        Params
        ------
        data: pd.Series, pd.DataFrame, dict, sequence
            If dict: should contain the field 'date' in a datetime
            format to be plotted. If not a numerical ordinal sequence
            will be used.
        column: str
            Column or field from which select the timeseries. This is required
            for pd.Series, pd.DataFrame, dict formats.

        Returns
        -------
        x: sequence or array-like
            Sequence with dates or numbers corresponding to the timeseries
            steps.
        y: sequence or array-like
            Sequence with the time series to be plotted.

        """
        if isinstance(data, pd.Series):
            return data.index.copy(), data.copy()
        elif isinstance(data, (pd.DataFrame, dict)):
            if column in data:
                y = data[column]
            else:
                raise(ValueError("Selected column: '%s'" % column +
                                 " not in 'data' variable of type" +
                                 " %s." % type(data)))
            if 'date' in data:
                x = convert_to_datetime(data['date'])
            else:
                if hasattr(data, 'index'):
                    x = data.index
                else:
                    x = np.arange(len(y))
            return x, y
        else:
            y = data
            x = np.arange(len(y))
            return x, y

    def format_data(self, data):
        """
            Check if data is a valid format and give required
            format if valid.
        """
        # Check data is valid type:
        # (pd.DataFrame, pd.Series, list, dict, np.ndarray)
        self.__is_valid_type(data)
        result = self._format(data)
        return result, self.names

    def format_input_data(self, input_data, column='adj_close'):
        assert isinstance(input_data, (dict, list)), (
            "Data should be contained in 'dict' object or 'list'")
        if isinstance(input_data, list):
            result = {"plot_" + str(i): v for i, v in enumerate(input_data)}
        else:
            result = input_data

        names = {}
        dict_total = {}
        _temp = {}
        for j, (plot_title, data) in enumerate(result.items()):
            _temp[plot_title], names[plot_title] = self.format_data(data)
            dict_total[plot_title] = {}
            for i, stock in enumerate(_temp[plot_title]):
                _, y = self._get_x_y(stock, column)
                dict_total[plot_title][names[plot_title][i]] = y
        df_total = {k: pd.concat(l, axis=1)
                    for k, l in list(dict_total.items())}
        # it's in __process_dict
        x_range = pd.concat(copy.deepcopy(df_total), axis=1).index
        formatted_data = self.__process_dict(df_total)

        formatted_result = {}
        for j, (plot_title, v) in enumerate(list(names.items())):
            _temp = {}
            for i, name in enumerate(v):
                _temp[name] = formatted_data[j][name]
            formatted_result[plot_title] = list(_temp.values())
        return formatted_result, x_range, names

    @staticmethod
    def _get_input_params(i, data, plot_title, params, data_dim):
        _params = {}
        if isinstance(params, dict):
            if plot_title in params:
                _params = copy.deepcopy(params[plot_title])
        elif isinstance(params, list):
            assert(data_dim == len(params)), "If input data contains " + \
                "a list, 'params' should contain a list of parameters" + \
                " for each element."
            _params = copy.deepcopy(params[i])
        return _params

    def format_params(self, input_data, params, names):
        if not isinstance(params, (dict, list)):
            raise(TypeError("'params' should be either 'dict' or 'list."))
        _params = {}
        _params = {plot_title: self._get_input_params(
            i,
            input_data[plot_title],
            plot_title,
            params, len(input_data))
            for i, (plot_title, stocksnames) in enumerate(names.items())}
        return _params

    def format_aligment(self, aligment, names):
        """
            Reviews input aligment setting to match the available 'names'
            structure and in case there is no parameter, set the
            stock to be at the left.
        """
        if not isinstance(aligment, (dict)):
            raise(TypeError("'aligment' should be either 'dict' in the form" +
                            "\{plot_title : \{stockname: 'left', ...\}\}"))
        for plot_title, _names in list(names.items()):
            if plot_title not in aligment:
                aligment[plot_title] = {}
            for stockname in _names:
                try:
                    if (not aligment[plot_title][stockname] or
                        not aligment[plot_title][stockname]
                            in ('left', 'right', None)):
                        aligment[plot_title][stockname] = None
                except:
                    aligment[plot_title][stockname] = None
            # remove keys that are not present in data
            for alig_name in list(aligment[plot_title].keys()):
                if alig_name not in _names:
                    del(aligment[plot_title][alig_name])
        # Order the dict to match the order in 'names'.
        # The order in dictionary elements corresponds to the order
        # of adding the element.
        result = {}
        for plot_title, _names in list(names.items()):
            result[plot_title] = {}
            for stockname in _names:
                result[plot_title][stockname] = aligment[plot_title][stockname]
        return result

    def format_y_label_right(self, y_label_right, y_label, names):
        if not isinstance(y_label_right, (dict)):
            raise(TypeError("'ylabel_right' should be either 'dict'" +
                            " in the form {plot_title : y_label_right}"))
        for plot_title in list(names.keys()):
            # if there is only one element in the plot (plot_title)
            # and there is no label for it, copy left y_label
            # in case there is need to alight to the right
            if len(names[plot_title]) == 1:
                y_label_right[plot_title] = y_label
            elif plot_title not in y_label_right:
                y_label_right[plot_title] = None
        return y_label_right



#!/usr/bin/python
# Authors: Mabel Villalba Jimenez <mabelvj@gmail.com>,
#          Emilio Molina Martinez <emilio.mol.mar@gmail.com>
# License: GPLv3

"""
    Example of usage.
    To run do in the command line: bokeh serve main.py
"""
import numpy as np
import pandas as pd
from stocksdashboard import *
from stocksdashboard import StocksDashboard
import sys
print(sys.path)
import pdb
pdb.set_trace()
from dashboard_with_widgets import DashboardWithWidgets
from bokeh.sampledata.stocks import AAPL
from bokeh.layouts import row, widgetbox
from bokeh.io import curdoc
from bokeh.server.server import Server


column = 'adj_close'

data = pd.DataFrame(AAPL[column], index=pd.date_range(
    start='1/1/2018', periods=len(AAPL[column])), columns=[column])
data_ema = data.ewm(span=int(
    np.round((252. / 12.) * 1.)),
    min_periods=1,
    adjust=True,
    ignore_na=False).mean()[column]

# Multiple formats for each line.
sdb = StocksDashboard()
sdb.build_dashboard(
    input_data={'stocks': {'STOCK': data},
                'ema': {'STOCK_ema': data_ema}},
    aligment={'stocks': {'STOCK': 'left'},
              'ema': {'STOCK_ema': None}},
    line_width=2.5, column=column, show=False)
layout = sdb.layout

dict_signals = {'STOCK_ema': "STOCK.ewm(span=w," +
                             "min_periods=1, " +
                             "adjust=True,ignore_na=False)" +
                             ".mean()"}
dbw = DashboardWithWidgets(
    sdb,
    sliders_params={'w': {'title': 'EMA window',
                          'params': {
                              'value': 20,
                              'start': 2,
                              'end': 252,
                              'step': 5}
                          }},
    signals_expressions=dict_signals)
sliders = dbw.create_sliders()
dbw.widget_on_change()

plot_width = 980
tab_title = 'plot'

inputs = widgetbox(list(sliders.values()))

curdoc().add_root(row(inputs, layout, width=plot_width))
curdoc().title = tab_title



#!/usr/bin/env python3
# Authors: Mabel Villalba Jimenez <mabelvj@gmail.com>,
#          Emilio Molina Martinez <emilio.mol.mar@gmail.com>
# License: GPLv3

try:
    from .formatter import Formatter
except Exception as excinfo:
    print(str(excinfo))
    from formatter import Formatter

import numpy as np
import pandas as pd
from bokeh.layouts import gridplot
from bokeh.plotting import figure
from bokeh.models import ColumnDataSource
from bokeh.models import Range1d
from bokeh.models.ranges import DataRange1d

from bokeh.models import LinearAxis
from bokeh.models import Axis
from bokeh.models.glyphs import Line
from bokeh.core.properties import value
import warnings
import copy

from bokeh.models import HoverTool
from bokeh.io import curdoc
from bokeh.layouts import row, widgetbox
import bokeh

WIDTH = 1024
HEIGHT = 648


def convert_to_datetime(x):
    return np.array(x, dtype=np.datetime64)


def get_colors(number_of_colors, palette_name='Category20'):
    from bokeh.palettes import all_palettes
    url_palettes = 'https://bokeh.pydata.org/en/' + \
                   'latest/docs/reference/palettes.html'
    warnings.warn("""Using palette %s.
        For other palettes visit:
        %s""" % (palette_name, url_palettes))
    colors = all_palettes[palette_name][max(number_of_colors, 3)]
    return colors


class StocksDashboard():
    tooltips = [
        ('date', '$x{%F}'),
        ('value', '$y{0.000}')
    ]
    formatters = {
        '$x': 'datetime',
    }
    mode = 'vline'
    names = None
    datasources = []

    def __init__(self, width=WIDTH, height=HEIGHT, ncols=1):
        self.width = width
        self.height = height
        self.ncols = ncols
        self._check_variables()

    def _check_variables(self, varname=None):

        def _is_valid_type(__varname=None, __value=None):
            if not __value:
                msg = "'%s cannot be None.'" % __varname
                raise(ValueError(msg))
            elif not isinstance(__value, int):
                msg = "'%s cannot be of" % __varname + \
                      "type %s, must be 'int'" % type(__value)
                raise (TypeError(msg))
            else:
                return True

        if varname and hasattr(self, varname):
            return _is_valid_type(varname, getattr(self, varname))
        else:
            for _varname, _value in self.__dict__.items():
                if _varname in ['width', 'height', 'ncols']:
                    _is_valid_type(_varname, _value)

    @staticmethod
    def _create_hover(tooltips=[('date', '$x{%F}'), ('value', '@y{0.000}')],
                      formatters={'$x': 'datetime'}, mode='vline', **kwargs):
        hover = HoverTool(**kwargs)
        hover.tooltips = tooltips
        hover.formatters = formatters
        hover.mode = mode
        return hover

    def _update_params(self, params, kwargs={}, names=None, aligment={}):
        # TODO: remove kwargs parameter
        """
            Update the aesthetics for plotting. Combine params and kwargs.

            Parameters
            ----------
            params: dict or dict of dicts
                Dict with params or dict with names and params per name.
            kwargs: dict
                Dict with general params for plotting.
            names: list or sequence of strings, default None
                List of names to be plotted.
            aligment: dict of strings
                Contains the axis to be selected for the line. Values
                for each dict can be: ('left', 'right', None).

            Returns
            -------
            result: dict
                Dict containing the parameters to use in the plotting.
                For more details see :Examples:

            Examples
            --------

            If params does not contains names:
                - Combine them:
                >>> import StocksDashboard as sdb
                >>> sdb.update_params(params = {'line_style': 'dot',
                ...                             'color': 'blue'},
                ...                   kwargs = {'line_width': 1.5})
                {'color': 'blue', 'line_style': 'dot', 'line_width': 1.5}
            If params contains names:
                - Update each dict:
                >>> sdb.update_params(params = {'GOOGL': {'line_style': 'dot'},
                ...                             'AAPL' :{'color': 'blue'}},
                ...                   kwargs = {'line_width': 1.5},
                ...                   names = ['GOOGL', 'AAPL'])
                {'AAPL': {'color': 'blue', 'line_width': 1.5},
                'GOOGL': {'line_style': 'dot', 'line_width': 1.5}}
            If both ``params`` and ``kwargs`` are empty, ``kwargs``
            is returned.
                >>> sdb.update_params(params = {}, kwargs={})
                {}
        """
        if params or kwargs:
            if (not names or
                (names and not any([n in params for n in names]) and not
                    aligment)):
                params.update(copy.deepcopy(kwargs))
            else:
                _initial_params = {}
                if params and not all([n in params for n in names]):
                    # Params has parameters but not especific params per name.
                    _initial_params = copy.deepcopy(kwargs)
                    # Override kwargs with params for the plot
                    _initial_params.update(copy.deepcopy(
                        {k: v for k, v in list(params.items())
                         if k not in names}))
                    params = {}
                else:
                    _initial_params = copy.deepcopy(kwargs)

                for i, n in enumerate(names):
                    if n in params:
                        params[n].update(copy.deepcopy(_initial_params))
                    else:
                        params[n] = copy.deepcopy(_initial_params)
            if aligment:
                # only with more than one element we need two axis.
                # In the other case, we just move to right the main axis.
                for n in names:
                    if aligment[n] is 'right':
                        params[n].update(
                            {'y_range_name': self.y_right_name})
        else:
            if aligment:
                params = {n: {'y_range_name': self.y_right_name}
                          for n in names if aligment[n] is 'right'}
            else:
                params = copy.deepcopy(kwargs)
        return params

    @staticmethod
    def _add_color_and_legend(params, legend='', color='black'):
        _params = copy.deepcopy(params)
        if 'legend' not in _params:
            _params['legend'] = value(legend)
        if 'color' not in _params:
            _params['color'] = color
        return _params

    @staticmethod
    def __get_class_attr(params, class_object):
        if not isinstance(class_object, (list, tuple)):
            class_object = [class_object]
        # return {k: v for k, v in list(copy.deepcopy(params).items())
        #         if any([hasattr(c, k) for c in class_object])}
        result = {}
        for k, v in list(params.items()):
            if any([hasattr(c, k) for c in class_object]):
                result[k] = v
        return result

    def __get_kwargs_to_figure(self, _params):
        """
            From the input parameter for the current plot,
            select the ones that are properties of Figure
            and remove them from 'params' so the rest available
            are just params for Line.
        """
        params = copy.deepcopy(_params)
        kwargs_to_figure = self.__get_class_attr(params,
                                                 bokeh.plotting.figure())
        for k, v in list(kwargs_to_figure.items()):
            del(params[k])
        # bokeh.plotting.figure(**kwargs) creates a Figure
        # with the **kwargs but if not present,
        # the aethetics are not passed to the FigureOptions class
        # and hence they do not appear as element of the class.
        # https://bokeh.pydata.org/en/latest/_modules/bokeh/plotting/figure.html#Figure
        for k, v in list(params.items()):
            try:
                bokeh.plotting.figure(**{k: v})
                kwargs_to_figure[k] = copy.deepcopy(v)
                del(params[k])
            except Exception as excinfo:
                # print(str(excinfo))
                pass
        return kwargs_to_figure, params

    def __get_ranges(self, kwargs_to_figure, keyword='extra_y_ranges'):
        if keyword in kwargs_to_figure:
            value = kwargs_to_figure[keyword]
            del[kwargs_to_figure[keyword]]
        else:
            if hasattr(self, keyword):
                value = getattr(self, keyword)
            else:
                value = None
        return kwargs_to_figure, value

    def _get_params(self, params, name, color,
                    class_object=[bokeh.models.glyphs.Line,
                                  bokeh.core.property_mixins.ScalarLineProps]):
        """ Try to find specific parameters for a given name."""
        try:
            _params = self._add_color_and_legend(params[name], name, color)
        except (TypeError, KeyError):
            _params = self._add_color_and_legend(params, name, color)
        return _params

    def __update_datasource(self, datasource, stock, column, name):
        """
            Update the object datasource with data from each stock.
        """
        x, y = Formatter._get_x_y(stock, column)
        datasource.add(name=name, data=y)
        if 'x' not in datasource.data:
            datasource.add(name='x', data=x)
        return datasource

    def get_y_limits(self, data, aligment, position='right'):
        _min = None
        _max = None
        for i, (stockname, al) in enumerate(list(aligment.items())):
            if al == position:
                if _min:
                    _min = min(_min, np.nanmin(data[i]))
                else:
                    _min = np.nanmin(data[i])
                if _max:
                    _max = max(_max, np.nanmax(data[i]))
                else:
                    _max = np.nanmax(data[i])
        return _min, _max

    def set_limits(self, p, data, aligment, extra_y_ranges=None):

        try:
            # checks if 'right' is in aligment or not
            list(aligment.values()).index('right')
            if len(data) == 1:
                # There is only one element
                # and we want to align it to the right
                p.yaxis.visible = False
            self.y_right_name = 'y1'
            if not extra_y_ranges:
                y_limits_right = self.get_y_limits(data, aligment, 'right')
                p.extra_y_ranges = {self.y_right_name:
                                    Range1d(y_limits_right[0],
                                            y_limits_right[1])}
            else:
                p.extra_y_ranges = {
                    self.y_right_name: extra_y_ranges}
            # make sure that left limits are set to only signals in the left.
            y_limits_left = self.get_y_limits(data, aligment, 'left')
            p.xaxis.x_range = Range1d(y_limits_left[0], y_limits_left[1])
        except Exception as excinfo:
            # print(str(excinfo))
            pass

        return p

    def separate_Figure_and_Line_params(self, params, kwargs_to_bokeh):
        # Extract Figure attr from global settings dict kwargs_to_bokeh
        (kwargs_to_figure_general,
            kwargs_to_bokeh) = self.__get_kwargs_to_figure(kwargs_to_bokeh)

        # Extract Figure attr from the particular params for the plot.
        (kwargs_to_figure,
            params) = self.__get_kwargs_to_figure(params)

        # Extract right limits in case there are limits for the right plot
        (kwargs_to_figure,
         extra_y_ranges) = self.__get_ranges(kwargs_to_figure,
                                             'extra_y_ranges')
        kwargs_to_figure_general.update(kwargs_to_figure)
        _kwargs_to_figure = copy.deepcopy(kwargs_to_figure_general)

        return params, kwargs_to_bokeh, _kwargs_to_figure, extra_y_ranges

    def _plot_stock(self, data=None, names=None, p=None, column='adj_close',
                    title="Stock Closing Prices",
                    ylabel_right=None, add_hover=True,
                    params={}, aligment={}, height=None, **kwargs_to_bokeh):
        if not p:
            (params,
             kwargs_to_bokeh,
             kwargs_to_figure,
             extra_y_ranges) = self.separate_Figure_and_Line_params(
                params, kwargs_to_bokeh)
            p = figure(x_axis_type="datetime", title=title,
                       sizing_mode='scale_both', plot_width=self.width,
                       **kwargs_to_figure)
            if height:
                p.plot_height = int(height * self.height)
                # print(int(height*self.height))
            p.grid.grid_line_alpha = 0.3
            p.xaxis.axis_label = 'Date'
            p = self.set_limits(p, data, aligment, extra_y_ranges)

        # data, names = Formatter().format_data(input_data)
        colors = get_colors(len(data))
        params = self._update_params(params=params, kwargs=kwargs_to_bokeh,
                                     names=names, aligment=aligment)

        p_to_hover = []
        __datasource = ColumnDataSource()
        for i, stock in enumerate(data):
            __datasource = self.__update_datasource(__datasource, stock,
                                                    column, names[i])
            _params = self._get_params(params, names[i], colors[i])
            _p = p.line(x='x', y=names[i], source=__datasource, **_params)
            p_to_hover.append(_p)

        assert(len(__datasource.data) == len(data) + 1), (
            "Number of elements used as source don't match " +
            "data dimension.")  # len(data) + 1 -> all data and the x-axis
        self.datasources.append(__datasource)

        assert(len(p_to_hover) == len(data)), "Number of Lines " + \
                                              "don't match data dimension."
        # p.legend.orientation = "horizontal"
        p.legend.location = "top_left"
        p.legend.click_policy = "hide"
        if add_hover:
            hover = StocksDashboard._create_hover(self.tooltips,
                                                  self.formatters,
                                                  self.mode,
                                                  renderers=p_to_hover)
            p.add_tools(hover)
        try:
            # checks if 'right' is in aligment or not
            list(aligment.values()).index('right')
            p.add_layout(LinearAxis(y_range_name=self.y_right_name,
                                    axis_label=ylabel_right), 'right')
        except Exception as e:
            # warnings.warn(str(e))
            pass
        return p

    def build_dashboard(self,
                        input_data={},
                        aligment={},
                        params={},
                        title="stocks.py example",
                        ylabel='Price', ylabel_right={},
                        show=True,
                        column='adj_close',
                        height=[],
                        **kwargs_to_bokeh):
        plots = []
        _data, x_range, _names = Formatter().format_input_data(input_data,
                                                               column)
        _params = Formatter().format_params(_data, params, _names)
        _aligment = Formatter().format_aligment(aligment, _names)
        _y_label_right = Formatter().format_y_label_right(ylabel_right,
                                                          ylabel, _names)
        kwargs_to_bokeh['y_axis_label'] = ylabel
        if 'x_range' not in kwargs_to_bokeh:
            kwargs_to_bokeh['x_range'] = Range1d(x_range[0], x_range[-1])
        if not height:
            height = [(1. / len(_data))] * len(_data)
        else:
            assert len(height) == len(_data), (
                "Number of heights should be equal to the number of plots. " +
                "expected: %s, " % len(_data) +
                "found: %s, len(height)= %s. " % (height, len(height)))
            assert sum(height) == 1, (
                "All heights should sum up to 1, " +
                "found: %s, sum(height)=%s" % (height, sum(height)))
        for i, (plot_title, data) in enumerate(_data.items()):
            plots.append(self._plot_stock(
                data=data,
                names=_names[plot_title],
                title=plot_title,
                params=_params[plot_title],
                aligment=_aligment[plot_title],
                ylabel_right=_y_label_right[plot_title],
                height=height[i],
                ** kwargs_to_bokeh))

        layout = gridplot(plots,
                          plot_width=self.width,
                          ncols=self.ncols)
        self.layout = layout
        if show:
            curdoc().add_root(layout)
            curdoc().title = title
        return curdoc



"""
    Example of usage.
    To run do in the command line: bokeh serve main.py
"""

import numpy as np
import pandas as pd
from stocksdashboard import StocksDashboard
from bokeh.sampledata.stocks import AAPL, GOOG, IBM, MSFT


window_size = 30
window = np.ones(window_size) / float(window_size)
aapl = np.array(AAPL['adj_close'])
aapl_dates = np.array(AAPL['date'], dtype=np.datetime64)
aapl_avg = pd.DataFrame([aapl_dates,
                         np.convolve(aapl, window, 'same')],
                        index=['date', 'adj_close']).T

# Multiple formats for each line.
StocksDashboard().build_dashboard(
    input_data={'stocks': {'AAPL': AAPL, 'GOOG': GOOG,
                           'IBM': IBM, 'MSFT': MSFT},
                'avg': {'AAPL_avg': aapl_avg}},
    params={'stocks': {'GOOG': {'line_dash': 'dashed'},
                       'AAPL': {'color': 'blue'}},
            'avg': {'color': 'orange',
                    'line_width': 1.5}},
    line_width=2.5)



#! /usr/bin/env python

"""To build the package run:
>>> python setup.py bdist_wheel --universal

To upload it to pypitest:
>>> twine upload --repository-url https://test.pypi.org/legacy/ dist/*

>> pip install --index-url https://test.pypi.org/simple/ StocksDashboard
"""
import sys
import os
from os.path import dirname
import shutil
from distutils.command.clean import clean as Clean

if sys.version_info[0] < 3:
    import __builtin__ as builtins
    from ConfigParser import SafeConfigParser
else:
    import builtins
    from configparser import SafeConfigParser

config = SafeConfigParser()
path = config.read(os.path.join(os.path.abspath(
    dirname(__file__)), 'stocksdashboard', 'config.ini'))
config.read(path)  # All info about version and is there.
# MAINTAINER = "Mabel Villalba Jimenez"
# MAINTAINER_EMAIL = "mabelvj@gmail.com"
# VERSION = '0.10dev0'
# LICENSE = 'Creative Commons Attribution-Noncommercial-Share Alike license'
MAINTAINER = config['main']['MAINTAINER']
MAINTAINER_EMAIL = config['main']['MAINTAINER_EMAIL']
LICENSE = config['main']['LICENSE']
VERSION = config['main']['VERSION']
print(VERSION)


class CleanCommand(Clean):
    description = "Remove build artifacts from the source tree"

    def run(self):
        Clean.run(self)
        # Remove c files if we are not within a sdist package
        cwd = os.path.abspath(os.path.dirname(__file__))
        remove_c_files = not os.path.exists(os.path.join(cwd, 'PKG-INFO'))
        if remove_c_files:
            print('Will remove generated .c files')
        if os.path.exists('build'):
            shutil.rmtree('build')
        for dirpath, dirnames, filenames in os.walk('stocksdashboard'):
            for filename in filenames:
                if any(filename.endswith(suffix) for suffix in
                       (".so", ".pyd", ".dll", ".pyc")):
                    os.unlink(os.path.join(dirpath, filename))
                    continue
                # extension = os.path.splitext(filename)[1]
                # if remove_c_files and extension in ['.c', '.cpp']:
                #     pyx_file = str.replace(filename, extension, '.pyx')
                #     if os.path.exists(os.path.join(dirpath, pyx_file)):
                #         os.unlink(os.path.join(dirpath, filename))
            for dirname in dirnames:
                if dirname == '__pycache__':
                    shutil.rmtree(os.path.join(dirpath, dirname))


cmdclass = {'clean': CleanCommand}


def setup_package():
    metadata = dict(
        name='stocksdashboard',
        version=VERSION,
        description='A package to build Stocks Dashboards in Bokeh',
        maintainer=MAINTAINER,
        maintainer_email=MAINTAINER_EMAIL,
        author='%s, Emilio Molina Martinez' % MAINTAINER,
        author_email="%s, emilio.mol.mar@gmail.com" % MAINTAINER_EMAIL,
        keywords=['bokeh', 'stocks', 'dashboard'],
        packages=['stocksdashboard', ],
        license=LICENSE,
        url='https://github.com/stocksdashboard/stocks_dashboard_bokeh',
        # install_requires=requirements,
        cmdclass=cmdclass,
        # long_description=open('README.txt').read(),
        include_package_data=True,
    )
    # if len(sys.argv) == 1 or (
    #         len(sys.argv) >= 2 and ('--help' in sys.argv[1:] or
    #                                 sys.argv[1] in ('--help-commands',
    #                                                 'egg_info',
    #                                                 '--version',
    #                                                 'clean'))):

    try:
        from setuptools import setup
    except ImportError:
        from distutils.core import setup

    with open('requirements.txt') as f:
        requirements = f.read().splitlines()
        metadata['install_requires'] = requirements

    setup(**metadata)


if __name__ == "__main__":
    setup_package()



from .stocksdashboard import StocksDashboard
from .stocksdashboard import convert_to_datetime
from .stocksdashboard import get_colors
from .formatter import Formatter
from .dashboard_with_widgets import DashboardWithWidgets

import sys
import os
from os.path import dirname

if sys.version_info[0] < 3:
    from ConfigParser import SafeConfigParser
else:
    from configparser import SafeConfigParser

__all__ = ['StocksDashboard', 'convert_to_datetime', 'get_colors', 'Formatter',
           'DashboardWithWidgets']

config = SafeConfigParser()
path = config.read(os.path.join(os.path.abspath(
    dirname(__file__)), 'config.ini'))
config.read(path)
__author__ = config['main']['AUTHORS']
__license__ = config['main']['LICENSE']
__version__ = config['main']['VERSION']
